#!/usr/bin/env python
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#cmd message class
#Essential Pathways v1.0
#Author:realasking
#Email:realasking@gmail.com,tomwangsim@163.com
#Aug-06,2017,in USTB
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
import sys 
import os
import re
import shutil

class ops:
    def __init__(self):
        pass 

    def module_backup(self,df,bf):
        shutil.copy(df,bf)

    def module_restore(self,bf,df):
        shutil.copy(bf,df)

    def uninstall(self,module_name,module_file):
        os.system('module unload '+module_name)
        #print(module_name)
        #print(module_file)
        shutil.rmtree(module_file,ignore_errors=True)
        os.remove(module_file)
  
class warnings:
    def __init__(self):
        pass 

    def Inerror(self):
        print("Input command not supported.\n")
    
    def Merror(self):
        print('Please install tcl version Environment Modules, and then setup your custom modules\' path using $HOME/.modulespath file\n')

    def Mnerror(self):
        print('Please define a writable path to save your private modules\n')

    def Perror(self):
        print("Input folder path not exist.\n")

    def Pderror(self):
        print("The pathway is not defined yet.\n")

    def Outhelp(self):
        print("Essential PATHWAY v1.0\n")
        print("This script is released under the GPLv3 License\n")
        print("Author:Zheng Wang <realasking@qq.com,tomwangsim@163.com>\n")
        print("2017-08-06\n")
        print("Usage:\n")
        print("      *Quckly add current foler to a pathway:")
        print("       $ep a PATHWAY_NAME\n")
        print("      *Listing exsit pathways:")
        print("       $ep l\n")
        print("      *Create a pathway:")
        print("       $ep a PATHWAY_NAME PATH_TO_YOUR_FOLDER\n")
        print("      *Make or change a short comment for an exists pathway:")
        print("       $ep c PATHWAY_NAME\n")
        print("      *Modify a pathway:")
        print("       $ep m PATHWAY_NAME PATH_TO_NEW_FOLDER\n")
        print("      *Delete a pathway:")
        print("       $ep d PATHWAY_NAME\n")
        print("      *Check paths\' availabilities and automatic delete invalid ones:")
        print("       $ep k\n")
        print("      *Refresh the Env-module:")
        print("       $ep r\n")
        print("      *Backup settings:")
        print("       $ep b\n")
        print("      *Uninstall all settings:")
        print("       $ep u\n")




#!/usr/bin/env python
__all__=["cmds","displayops","einit","epprocess"]
from . import cmds 
from . import displayops 
from . import einit 
from . import epprocess

__name__='essential-pathway'
__version__ = '1.0'
__short_description__ = 'Manage your most commonly used directories in command line, requires the tcl edition Environment Module.'
__author__ = 'realasking'
__author_email__ ='<realasking@gmail.com,tomwangsim@163.com>'
__license__ = 'LGPLv3'



#!/usr/bin/env python
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#A class for display properties
#Essential Pathways v1.0
#Author:realasking
#Email:realasking@gmail.com,tomwangsim@163.com
#Aug-06,2017,in USTB
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
import sys
import os
import re
import shutil

class screenformat:
      max_tty_rows=0
      max_tty_line_width=0
      fullptlength=0
      namelengthmax=0
      waylengthmax=0

      def __init__(self):
          arows,awidth=os.popen('stty size', 'r').read().split() 
          self.max_tty_rows=int(arows)
          self.max_tty_line_width=int(awidth)
    
      #obtain tty width, and set columns' widths automatically
      def size_calc(self,fileds):
          self.fullptlength=self.max_tty_line_width-3*int(fileds)-1
          self.namelengthmax=int(0.3*self.fullptlength)
          self.waylengthmax=int((self.fullptlength-self.namelengthmax)/(int(fileds)-1.0))

      #formatting strings to fit table length
      def string_formatting(self,data,filed_max_width):
          all_length=0
          fdata=""
          for cha in data:
               if all_length+2<filed_max_width:
                   fdata=fdata+cha 
                   all_length=all_length+1
               else:
                   all_length=1
                   fdata=fdata+"\n"+cha 
          return fdata




#!/usr/bin/env python
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#Process the database for essential pathways
#Essential Pathways v1.0
#Author:realasking
#Email:realasking@gmail.com,tomwangsim@163.com
#Aug-06,2017,in USTB
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
import sys
import os
import re
import sqlite3
import shutil
from epw import displayops as dpops
from epw import cmds
from epw import einit as ei
from prettytable import PrettyTable

__version__ = '1.0'
__author__ = 'realasking <realasking@gmail.com,tomwangsim@163.com>'
__license__ = 'LGPLv3'

class pathwayops(ei.einit):
      #def __init__(self,df,mn,mf): 
      def __init__(self,conf_folder,dbfile,module_name):
          ei.einit.__init__(self,conf_folder,dbfile,module_name)
          if not os.path.isfile(self.df):
             self.epwdb=sqlite3.connect(self.df)
             self.db=self.epwdb.cursor() 
             self.db.execute('''CREATE TABLE ep(pathway_name text,folder_path text,comment text)''') 
             self.epwdb.commit()
          else:
             self.epwdb=sqlite3.connect(self.df)
             self.db=self.epwdb.cursor()
          #self.module_name=mn
          #self.module_file=mf
          self.info=cmds.warnings()
          self.cmds=cmds.ops()
      
      def close(self):
             self.epwdb.close()

      def dbdelete(self,pn):
             self.db.execute('DELETE FROM ep WHERE pathway_name=?',(pn,))
             self.epwdb.commit()

      def dbcheck(self):
          #Please unload module epath in system before use this function
          self.db.execute('DELETE FROM ep WHERE rowid NOT IN (SELECT min(rowid) FROM ep GROUP BY folder_path)')
          self.epwdb.commit()
          self.db.execute('DELETE FROM ep WHERE rowid NOT IN (SELECT min(rowid) FROM ep GROUP BY pathway_name)')
          self.epwdb.commit()
          #Delete the pathways which conflict with other environment variables of the system
          for i in os.popen('env|cut -d\"=\" -f1').read().splitlines():
                self.dbdelete(i)
    
      def module_create(self):
          fp=open(self.module_file,'w')
          fp.write("#%Module1.0"+'\n')
          fp.write("               ## Module settings for essential pathways \n")
          fp.write("               ## This file is automatically generated by ep, do not modify it manually\n")
          fp.write("               proc ModulesHelp { } {\n")
          fp.write("                   puts stderr \"Essential Pathways are defined in this file,do not modify it manually\"\n")
          fp.write("               }\n")
          fp.write("\n")
          fp.write("               module-whatis \"Essential Pathways\"\n")
          self.dbcheck()
          for row in self.db.execute('SELECT * FROM ep ORDER BY pathway_name'):
              fp.write("               setenv "+row[0]+" \""+row[1]+"\"\n")    
          fp.close()

      #In consideration
      def path_mod(self):
          #remove duplicates in db 
          self.dbcheck()
          #Check if module file exists
          if os.path.isfile(self.module_file):
             os.remove(self.module_file)

      def path_refresh(self):
          #self.dbcheck()
          #Check if module file exists
          if os.path.isfile(self.module_file):
          #Check if module file has pathways undfined in db
            for i in os.popen('module show '+self.module_name+' |grep setenv|awk \'{for(i=2;i<=NF;i++) printf $i" ";print""}\'').read().splitlines():
                  #split a string at the first blank
                  sbuff=i.strip().split(' ',1)
                  j=self.db.execute('SELECT * FROM ep WHERE pathway_name=?',(sbuff[0],))
                  if len(j.fetchall())==0:
                     self.db.execute('''INSERT INTO ep VALUES(?,?,?)''',(sbuff[0],sbuff[1],''))
                     self.epwdb.commit()
                  else:
                     for k in j.fetchall():
                         if sbuff[1]!=k[1]:
                            self.db.execute('UPDATE ep SET folder_path=? WHERE pathway_name=?',(sbuff[1],sbuff[0]))
                            self.epwdb.commit()
          os.remove(self.module_file)
          self.module_create()
          self.close()

      def path_list(self):
          print('Defined PATHWAYS:')
          screeninfo=dpops.screenformat()
          screeninfo.size_calc(3)
          #fplen,namelen,waylen=screeninfo.size_calc(3)
          t1 = PrettyTable(['PATHWAY NAME', 'FOLDER PATH', 'COMMENT'])
          t1.align['PATHWAY NAME']='c'
          t1.padding_width = 1
          for row in self.db.execute('SELECT * FROM ep ORDER BY pathway_name'):
              row_1=screeninfo.string_formatting(row[0],screeninfo.namelengthmax)
              row_2=screeninfo.string_formatting(row[1],screeninfo.waylengthmax)
              row_3=screeninfo.string_formatting(row[2],screeninfo.waylengthmax)
              t1.add_row([row_1,row_2,row_3])
          print(t1)
          screeninfo2=dpops.screenformat()
          screeninfo2.size_calc(2)
          #fplen,namelen,waylen=screeninfo2.size_calc(2)
          print('Set PATHWAYS:')
          t2 = PrettyTable(['PATHWAY NAME', 'FOLDER PATH'])
          t2.align['PATHWAY NAME']='c'
          t2.padding_width = 1 
          for row1 in os.popen('module show '+self.module_name+' |grep setenv|awk \'{for(i=2;i<=NF;i++) printf $i" ";print""}\'').read().splitlines():
              tbuff=row1.strip().split(' ',1)
              row1_1=screeninfo2.string_formatting(tbuff[0],screeninfo2.namelengthmax)
              row1_2=screeninfo2.string_formatting(tbuff[1],screeninfo2.waylengthmax)
              t2.add_row([row1_1,row1_2]) 
          print(t2)
          self.close()

      def path_create(self,pn,pw='',pc=''):
          self.path_mod()
          #print(pn)
          if not os.path.exists(pw):
             self.info.Perror()
             exit()
          j=self.db.execute('SELECT * FROM ep WHERE pathway_name=?',(pn,))
          if len(j.fetchall())==0:
             self.db.execute('INSERT INTO ep VALUES(?,?,?)',(pn,pw,pc))
             self.epwdb.commit()
          self.module_create()
          self.close()

      def path_comment(self,pn):
          j=self.db.execute('SELECT * FROM ep WHERE pathway_name=?',(pn,))
          if len(j.fetchall())!=0:
             print("Pathway name: "+pn+"\n")
             print("Exsits comment:\n")
             for row in self.db.execute('SELECT * FROM ep WHERE pathway_name=?',(pn,)):
                 print(row[2])
             pc=input("New comment (Ended by Enter):\n")
             self.db.execute('UPDATE ep SET comment=? WHERE pathway_name=?',(pc,pn))
             self.epwdb.commit() 
          else:
             self.info.Pderror()
          self.close()

      def path_modify(self,pn,pw):
          if not os.path.exists(pw):
             self.info.Perror()
             self.epwdb.close()
          else:
             j=self.db.execute('SELECT * FROM ep WHERE pathway_name=?',(pn,))
             if len(j.fetchall())!=0:
                self.db.execute('UPDATE ep SET folder_path=? WHERE pathway_name=?',(pw,pn))
                self.epwdb.commit()
             else:
                self.info.Pderror()
             self.module_create()
             self.close()

      def path_delete(self,pn):
          self.dbdelete(pn)
          self.module_create()
          self.close()

      def cmodfile(self):
          if not os.path.isfile(self.module_file):
             self.module_create()
      
      def path_backup(self):
          self.cmds.module_backup(self.df,self.bf)

      def path_restore(self):
          self.cmds.module_restore(self.bf+'/'+self.dbfile,self.df)
          self.module_create()
          self.close()
      
      def path_uninstall(self):
          self.close()
          self.cmds.uninstall(self.module_name,self.module_file)




#!/usr/bin/env python
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#einit
#Essential Pathways v1.0
#Author:realasking
#Email:realasking@gmail.com,tomwangsim@163.com
#Aug-06,2017,in USTB
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
import sys
import os
import re
import sqlite3
import shutil
from epw import cmds

class einit():
      def __init__(self,cfolder,dbf,mname):
          self.home=os.path.expanduser("~")
          #cfolder should be .ep 
          self.conf_folder=cfolder 
          #dbf should be ep.db
          self.dbfile=dbf
          #mname should be epath
          self.module_name=mname
          self.folder=self.home+'/'+self.conf_folder
          self.df=self.folder+'/'+self.dbfile
          self.bf=self.home+'/BEP'
          self.info=cmds.warnings()

          if not os.path.exists(self.folder):
             os.makedirs(self.folder,exist_ok=True)

          if not os.path.exists(self.bf):
             os.makedirs(self.bf,exist_ok=True)

          if not os.path.isfile(self.home+'/.modulespath'):
             self.info.Merror()
             exit()
          else:
             setmf=0
          for modules_folder in re.split(':',os.environ['MODULEPATH']):
              if os.access(modules_folder, os.W_OK):
                  setmf=1
                  self.module_file=modules_folder+'/'+self.module_name
                  break
          if setmf==0:
             self.info.Mnerror()
             exit()




#!/usr/bin/env python
from setuptools import setup, find_packages

with open('README.md') as buff:
    longdes = buff.read()

with open('changelog.md') as buff:
    longdes += buff.read()

CLASSIFIERS = [
    'Programming Language :: Python :: 3',
    'Topic :: Utilities',
    'Topic :: Education',
    'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
    'Development Status :: 5 - Production/Stable'
]

setup(
    name='essential-pathway',
    version='1.0',
    keywords=['ep','epw','environment variables','path','pathway','essential pathway'],
    description='Manage your most commonly used directories in command line, requires the tcl edition Environment Module.',
    long_description=longdes,
    author='realasking',
    author_email='realasking@gmail.com,tomwangsim@163.com',
    url='https://github.com/realasking/essential-pathway',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['prettytable'],
    scripts=['bin/ep'],
    platforms=['linux','cygwin','freebsd','Unix'],
    classifiers=CLASSIFIERS,
)
